@@ -94,8 +94,12 @@ class ConverterRegistry:
         c = converter_type()
         if c.type in self._converters:
             raise ValueError(converter_type)
+        # bool, <type 'bool'>, 'bool'
         self._converters[c.type] = c
-
+        self._converters[str(c.type)] = c
+        self._converters[c.type.__name__] = c
+        return c
+    
     def remove(self, converter_type):
         """
         Removes converter_type from the registry
@@ -111,19 +115,21 @@ class ConverterRegistry:
         del self._converters[ctype]
 
     def get_converter(self, converter_type):
-        # This is a hack:
-        # If we're a subclass of enum, create a dynamic subclass on the
-        # fly and register it, it's necessary for enum.from_string to work.
-        if (issubclass(converter_type, enum) and
-            not converter_type in self._converters):
-            self.add(type(enum.__class__.__name__ + 'EnumConverter',
-                       (_EnumConverter,), dict(type=converter_type)))
-
         try:
-            return self._converters[converter_type]
+            converter = self._converters[converter_type]
         except KeyError:
+            # This is a hack:
+            # If we're a subclass of enum, create a dynamic subclass on the
+            # fly and register it, it's necessary for enum.from_string to work.
+            if (issubclass(converter_type, enum) and
+                not converter_type in self._converters):
+                return self.add(
+                    type(enum.__class__.__name__ + 'EnumConverter',
+                         (_EnumConverter,), dict(type=converter_type)))
             raise KeyError(converter_type)
 
+        return converter
+
     def get_converters(self, base_classes=None):
         if base_classes is None:
             return self._converters.values()
@@ -136,27 +142,21 @@ class ConverterRegistry:
             base_classes = tuple(base_classes)
             converters.append(self._converters[object])
 
-        for datatype in self._converters:
-            if issubclass(datatype, base_classes):
-                converters.append(self._converters[datatype])
+        for converter in self._converters.values():
+            if issubclass(converter.type, base_classes):
+                converters.append(converter)
 
         return converters
 
     def check_supported(self, data_type):
-        value = None
-        for t in self._converters.values():
-            if t.type == data_type or t.type.__name__ == data_type:
-                value = t.type
-                break
+        converter = self._converters.get(data_type)
+        if converter is None:
+            supported = ', '.join(map(str, self._converters.keys()))
+            raise TypeError(
+                "%s is not supported. Supported types are: %s"
+                % (data_type, supported))
 
-        assert not isinstance(value, str), value
-
-        if not value:
-            type_names = [t.type.__name__ for t in self._converters.values()]
-            raise TypeError("%s is not supported. Supported types are: %s"
-                            % (data_type, ', '.join(type_names)))
-
-        return value
+        return converter.type
 
     def as_string(self, converter_type, value, format=None):
         """
@@ -209,12 +209,15 @@ class BaseConverter(object):
 
     def get_compare_function(self):
         """
-        @returns:
+        This can be overriden by a subclass to provide a custom comparison
+        function.
+        @returns: cmp
         """
         return cmp
 
     def as_string(self, value, format):
         """
+        Convert the value to a string using the specificed format.
         @param value:
         @param format:
         @returns:
@@ -222,13 +225,15 @@ class BaseConverter(object):
 
     def from_string(self, value):
         """
+        Convert a value from a string.
         @param value:
         @returns:
         """
 
     def get_mask(self):
         """
-        @returns:
+        Returns the mask of the entry or None if not specified.
+        @returns: the mask or None.
         """
         return None
 
@@ -316,7 +321,8 @@ class _BoolConverter(BaseConverter):
         elif value.upper() in ('FALSE', '0'):
             return False
 
-        return ValidationError(_("'%s' can not be converted to a boolean") % value)
+        return ValidationError(
+            _("'%s' can not be converted to a boolean") % value)
 
 converter.add(_BoolConverter)
 
@@ -515,6 +521,11 @@ class _BaseDateTimeConverter(BaseConverter):
         if value is None:
             return ''
 
+        if isinstance(value, (datetime.date, datetime.datetime)):
+            if value.year < 1900:
+                raise ValidationError(
+                    _("You cannot enter a year before 1900"))
+
         return value.strftime(format)
 
     def _convert_format(self, format):
@@ -539,12 +550,19 @@ class _BaseDateTimeConverter(BaseConverter):
             # time.strptime (python 2.4) does not support %r
             # pending SF bug #1396946
             dateinfo = time.strptime(value, format)
-            return self.from_dateinfo(dateinfo)
+            date = self.from_dateinfo(dateinfo)
         except ValueError:
             raise ValidationError(
                 _('This field requires a date of the format "%s" and '
                   'not "%s"') % (self._convert_format(format), value))
 
+        if isinstance(date, (datetime.date, datetime.datetime)):
+            if date.year < 1900:
+                raise ValidationError(
+                    _("You cannot enter a year before 1900"))
+        return date
+
+
 class _TimeConverter(_BaseDateTimeConverter):
     type = datetime.time
     name = _('Time')
