@@ -113,7 +113,7 @@ class SignalBroker(object):
             match = method_regex.match(fname)
             if match is None:
                 continue
-            after, w_name, signal = match.groups()
+            on_after, w_name, signal = match.groups()
             widget = getattr(view, w_name, None)
             if widget is None:
                 raise AttributeError("couldn't find widget %s in %s"
@@ -125,11 +125,13 @@ class SignalBroker(object):
             # Must use getattr; using the class method ends up with it
             # being called unbound and lacking, thus, "self".
             try:
-                if after:
+                if on_after == 'on':
+                    signal_id = widget.connect(signal, methods[fname])
+                elif on_after == 'after':
                     signal_id = widget.connect_after(signal, methods[fname])
                 else:
-                    signal_id = widget.connect(signal, methods[fname])
-            except TypeError:
+                    raise AssertionError
+            except TypeError, e:
                 raise TypeError("Widget %s doesn't provide a signal %s" % (
                                 widget.__class__, signal))
             self._autoconnected.setdefault(widget, []).append((
@@ -486,11 +488,13 @@ class SlaveView(gobject.GObject):
     # Slave handling
     #
 
-    def attach_slave(self, name, slave):
+    def attach_slave(self, name, slave, placeholder_widget=None):
         """Attaches a slaveview to the current view, substituting the
-        widget specified by name.  the widget specified *must* be a
-        eventbox; its child widget will be removed and substituted for
-        the specified slaveview's toplevel widget::
+        widget specified by placeholder_widget. If placeholder_widget is not
+        specified, an widget with the name specified must exist.
+
+        The widget specified *must* be a eventbox; its child widget will be
+        removed and substituted for the specified slaveview's toplevel widget::
 
          .-----------------------. the widget that is indicated in the diagram
          |window/view (self.view)| as placeholder will be substituted for the
@@ -528,7 +532,7 @@ class SlaveView(gobject.GObject):
         else: # slaveview
             new_widget = shell
 
-        placeholder = self.get_widget(name)
+        placeholder = placeholder_widget or self.get_widget(name)
         placeholder.set_data('kiwi::slave', self)
 
         if not placeholder:
